@@ -7,7 +7,10 @@
  */
 
 /** Load WordPress Administration Bootstrap */
-require_once('admin.php');
+require_once('./admin.php');
+
+if ( ! current_user_can('manage_links') )
+	wp_die(__('You do not have sufficient permissions to add links to this site.'));
 
 $title = __('Add New Link');
 $parent_file = 'link-manager.php';
@@ -21,7 +24,7 @@ wp_enqueue_script('link');
 wp_enqueue_script('xfn');
 
 $link = get_default_link_to_edit();
-include('edit-link-form.php');
+include('./edit-link-form.php');
 
-require('admin-footer.php');
-?>
\ No newline at end of file
+require('./admin-footer.php');
+?>
